package custom.curve;

import org.opensourcephysics.numerics.MultiVarFunction;
import org.opensourcephysics.numerics.LevenbergMarquardt;

/**
 *
 * @author danhurwitz
 */
public class NonLinearLSFit implements MultiVarFunction{

  double[] abscissa;
  double[] ordinate;
  double[] optimal;
  int iteration = 0; 
  
  public NonLinearLSFit(double[] abscissa, double[] ordinate){
    setData(abscissa, ordinate);
  }
  
  
  public void setData(double[] abscissa, double[] ordinate){
    this.abscissa = abscissa;
    this.ordinate = ordinate;
  }
  
  public void setGuess(double... x){
    this.optimal = x;
    
  }
  
  public double function(double xpt, double[] param){
  // ENTER FUNCTION HERE 
    
    double ret; 
//    ret = param[0]*xpt/(param[1] + xpt) ; // Michaelis-Menten
//    ret = Math.pow(xpt, param[0])/(param[1] + Math.pow(xpt, param[0])); // Hill
//    ret = param[0]/(param[1] + Math.exp(xpt));
    ret = param[0]*Math.pow((1 - Math.exp(-xpt/param[1]) ),param[2]); // Stretched Exponential
    return ret; 
    
  }
  
  
  public double evaluate(double[] x){
  // This argument x is parameter array. 
    this.optimal = x; 
    double err = 0; 
    for(int i = 0; i < this.abscissa.length; i++){
      err += Math.pow(ordinate[i] - function(abscissa[i], x),2);
    }   
    return err; 
    
  }
  
  public double[] getOptimalParameters(){
    return this.optimal;
  }
  
  public void findSolution(int niterations, double tolerance){
    LevenbergMarquardt lm = new LevenbergMarquardt();
    lm.minimize(this, optimal, niterations, tolerance);
//    System.out.println("Iterations " + lm.getIterations());
  }
  
  
  
 
}
