package custom.draw;

import java.awt.Color;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Random;

import javax.swing.JFrame;

import org.opensourcephysics.display.*;
import org.opensourcephysics.frames.PlotFrame;

public class DrawLibrary {


	////////////////// Box /////////////
	
	// List of boxes, default color 
	public static void drawBox(ArrayList<Rectangle2D> rectlist, DrawingPanel panel){
		
		for(int i = 0; i < rectlist.size(); i++){
			drawBox(rectlist.get(i), panel); 
		}
		
	}
	
	// List of boxes, once color
	public static void drawBox(ArrayList<Rectangle2D> rectlist, DrawingPanel panel, int[] color){
		
		for(int i = 0; i < rectlist.size(); i++){
			drawBox(rectlist.get(i), panel, color); 
		}
		
	}
	
	// List of boxes, two colors
	public static void drawBox(ArrayList<Rectangle2D> rectlist, DrawingPanel panel, int[] fcolor, int[] ecolor){
	
	  for(int i = 0; i < rectlist.size(); i++){
		drawBox(rectlist.get(i), panel, fcolor, ecolor); 
	  }
	
	}
	
	// One box, default color (blue, 150 opacity) 
	public static void drawBox(Rectangle2D box, DrawingPanel panel){
	DrawableShape bbox = DrawableShape.createRectangle(box.getX() + box.getWidth()/2, box.getY() + box.getHeight()/2, box.getWidth(), box.getHeight());
	Color fillColor = new Color(0, 255 , 0, 150);
	Color edgeColor = new Color(0, 255 , 0, 150);
	bbox.setMarkerColor(fillColor, edgeColor);
	panel.addDrawable(bbox);
	}
	
	// One box, one color
	public static void drawBox(Rectangle2D box, DrawingPanel panel, int[] color){
	  DrawableShape bbox = DrawableShape.createRectangle(box.getX() + box.getWidth()/2, box.getY() + box.getHeight()/2, box.getWidth(), box.getHeight());
	  Color fillColor = new Color(color[0], color[1] , color[2], color[3]);
	  Color edgeColor = new Color(color[0], color[1] , color[2], color[3]);
	  bbox.setMarkerColor(fillColor, edgeColor);
	  panel.addDrawable(bbox);
	}
	
	// One box, two colors
	public static void drawBox(Rectangle2D box, DrawingPanel panel, int[] fcolor, int[] ecolor){
	  DrawableShape bbox = DrawableShape.createRectangle(box.getX() + box.getWidth()/2, box.getY() + box.getHeight()/2, box.getWidth(), box.getHeight());
	  Color fillColor = new Color(fcolor[0], fcolor[1] , fcolor[2], fcolor[3]);
	  Color edgeColor = new Color(ecolor[0], ecolor[1] , ecolor[2], ecolor[3]);
	  bbox.setMarkerColor(fillColor, edgeColor);
	  panel.addDrawable(bbox);
     }
	
	
	////////////////// Cirlce /////////////////////
	
	
//  One circle, default color (red, opacity 255) 
	public static DrawableShape drawCircle(double[] point, double d){

	  DrawableShape p1 = DrawableShape.createCircle(point[0], point[1], d);
	  Color pfColor = new Color(255,0,0,255);
	  Color peColor = new Color(255,0,0,255);
	  p1.setMarkerColor(pfColor, peColor);
	  
	  return p1; 
	} // drawCircle method: one cell, default color
	
	
	
	
	
	
	// One circle, one color 
	public static DrawableShape drawCircle(double[] point, double d, int[] color){

	  DrawableShape p1 = DrawableShape.createCircle(point[0], point[1], d);
	  Color pfColor = new Color(color[0], color[1], color[2], color[3]);
	  Color peColor = new Color(color[0], color[1], color[2], color[3]);
	  p1.setMarkerColor(pfColor, peColor);
	  
	
	  return p1; 
	} // drawCircle method: one cell, one color 
	
	// One circle, two colors 
	public static DrawableShape drawCircle(double[] point, double d, int[] colorf, int[] colore){

	  DrawableShape p1 = DrawableShape.createCircle(point[0], point[1], d);
	  Color pfColor = new Color(colorf[0], colorf[1], colorf[2], colorf[3]);
	  Color peColor = new Color(colore[0], colore[1], colore[2], colore[3]);
	  p1.setMarkerColor(pfColor, peColor);
	  
	  return p1; 
	} // drawCircle method: one cell, two colors 
	
	// List of circles, default color 
	public static ArrayList<DrawableShape> drawCircle(ArrayList<double[]> circleList, double d, DrawingPanel panel){
	  ArrayList<DrawableShape> CircleList = new ArrayList<DrawableShape>(); 
		
	  for(int i = 0; i < circleList.size(); i++){
	    DrawableShape circle = drawCircle(circleList.get(i), d);
	    CircleList.add(circle);
	  }  
	  
	  return CircleList;
	} // drawCircle method: list of circles, default color  
	
	// List of circles, one color 
	public static ArrayList<DrawableShape> drawCircle(ArrayList<double[]> circleList, double d, int[] color){
	 ArrayList<DrawableShape> CircleList = new ArrayList<DrawableShape>(); 

	  for(int i = 0; i < circleList.size(); i++){
	    DrawableShape circle = drawCircle(circleList.get(i), d, color);
	    CircleList.add(circle);
	  }  
	  
	  return CircleList; 
	} // drawCircle method: list of circles, default color  
	
	// List of circles, two colors 
	public static ArrayList<DrawableShape> drawCircle(ArrayList<double[]> circleList, double d, int[] colorf, int[] colore){
		ArrayList<DrawableShape> CircleList = new ArrayList<DrawableShape>(); 
		
	  for(int i = 0; i < circleList.size(); i++){
	    DrawableShape circle = drawCircle(circleList.get(i), d, colorf, colore);
	    CircleList.add(circle); 
	  }  
	  
	  return CircleList; 
	} // drawCircle method: list of circles, default color  
	
	
	
	
	
	
	
	
	// Arrows 
	
	public static Arrow drawArrow(double[] pt, double[] dir, double L, Color color){
		Arrow arrow = new Arrow(pt[0], pt[1], L*dir[0], L*dir[1]);
		arrow.setColor(color);  
		return arrow; 
	} // end drawArrow
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	//////// Colors ///////////
	
	// Random color
		public static int[] getRandomColor(){
			Random random = new Random(); 
			int[] color = new int[3];
			for(int i = 0; i < 3 ; i++){
			  color[i] = (int) (255*random.nextDouble());
			}
				
		return color;	
		} // end random color method 
		
		
		
		// Get Rainbow
		public static ArrayList<Color> getRainbow(int numcolors){
		// Returns rainbow color map	
			ArrayList<Color> cmap = new ArrayList<Color>();
				for(int i = 0; i < numcolors; i++){
					float h = (i+1)*(1/(float)numcolors);
					cmap.add(Color.getHSBColor(h,1,1));
				}		
			return cmap;
		}
		
		
		
		
		/////// Plots ///////// 
		
		public static double[] hist(double[] data, int nbins){
			
			// Find min and max
			double min = data[0];
			double max = data[0];
			for(int i = 0; i < data.length; i++){
				if(data[i] < min){
					min = data[i];
				}
				if(data[i] > max){
					max = data[i]; 
				}
			} // min-max loop
			
			// Determine bin start values 
			double[] bins = new double[nbins];
			for(int i = 0; i < nbins; i++){
				bins[i] = min + i*(max-min)/(double)nbins;
			}
			
			// Sort data into bins
			double[] freq = new double[bins.length];
			for(int i = 0; i < data.length; i++){
				int cbin = 0;
				while(data[i] > bins[cbin] && cbin < bins.length-1){
					cbin++;
				}
				freq[cbin--]++;
			} // end for sort loop
			
			
			
			// Create plots 

			PlotFrame pframe = new PlotFrame("bins", "freq", "Histogram");
			Dataset dataset = new Dataset(); 
			dataset.append(bins, freq);
			pframe.append(0,bins,freq);
			pframe.setMarkerShape(0, Dataset.BAR);
			pframe.setVisible(true);
			pframe.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			return bins; 
			
		} // hist(double[]) (for 1d arrays of values) 
		
		
	public static double[] hist(double[][] data2d, int nbins){
		
		
		double[] data = new double[data2d.length*data2d[0].length];
		int index = 0; 
		for(int i = 0; i < data2d.length; i++){
			for(int j = 0; j < data2d[0].length; j++){ 
				data[index] = data2d[i][j];
				index++;
			}
		}
		
			double[] ret = hist(data, nbins);
			return ret; 
		} // End hist(double[][]) (for scalar fields)
		
	
	
	public static void plotter(double[][] data){
		
		// Get x and y values 
		double[] x = new double[data.length];		
		double[] y = new double[data.length];
		for(int i = 0; i < data.length  ; i++){
			x[i] = data[i][0];
			y[i] = data[i][1];
		} // end for loop
			
	plotter(x,y);

	} // end plotter double[][]	
	
	
	public static void plotter(double[] x, double[] y){
	
		PlotFrame pframe = new PlotFrame("x", "y", "Scatter");
		Dataset dataset = new Dataset(); 
		dataset.append(x, y);
		pframe.append(0,x,y);
		pframe.setMarkerShape(0, Dataset.CIRCLE);
		pframe.setVisible(true);
		pframe.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}	// end plotter 2xdouble[]
	
	public static void lineplot(double[] x, double[] y, Color color, String title){
		
//		DrawingPanel panel = new DrawingPanel();
		PlotFrame pframe = new PlotFrame("x", "y", title);
//		Dataset dataset = new Dataset(); 
//		dataset.append(x, y);
		pframe.append(0,x,y);
		pframe.setMarkerShape(0, Dataset.CIRCLE);
		pframe.setConnected(0,true);
		pframe.setLineColor(0,color); 
		pframe.setVisible(true);
		pframe.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}	// end plotter 2xdouble[]
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
}
