package custom.draw;

import org.opensourcephysics.display.*;
import org.opensourcephysics.display2d.*;
import org.opensourcephysics.frames.PlotFrame;
import java.awt.*;
import javax.swing.*;
import custom.draw.*;
import java.lang.Integer;
import java.util.ArrayList; 
import java.util.HashMap; 
import java.util.Random; 
import java.util.Iterator; 
import custom.array.*;
import java.beans.PropertyVetoException;

public class Plotter{

	
	public HashMap<Integer, Figure> figureList = new HashMap<Integer, Figure>(); 
	int nextfigure = 0; 
	boolean HOLD = false; 
	Figure cf; // cf = current_figure
	String CloseAction = "dispose"; 

	public Plotter(){} // end constructor 
	public void hold(boolean _hold){ this.HOLD = _hold ;} // end hold 

	public void setFigure(int fnum){ 
		if(!figureList.containsKey(new Integer(fnum))) figureList.put(fnum, new Figure()); 
		cf = figureList.get(fnum); 
	}

	private void setNextFigureIndex(){
		// Start at current nextfigure value, and increment until find an empty slot 
		while(figureList.containsKey(nextfigure)){
			nextfigure++; 
		}
	} // end getNextFigure() 
	
	public void show(){
		Iterator<Integer> keySetIterator = figureList.keySet().iterator(); 
		while(keySetIterator.hasNext()){
			figureList.get(keySetIterator.next()).setVisible(true); 
		}
	} // end show 

	public void show(Integer... ishow ){
		for(Integer i : ishow){
			figureList.get(i).setVisible(true); 
		} // for loop 
	} // end show 
	
	public void title(String _title){
		checkForEmptyList();
		cf.title(_title);
	} // end title 
	public void xaxis(String _xlab){
		checkForEmptyList();
		cf.xaxis(_xlab);
	} // end xaxis
	public void yaxis(String _ylab){	
		checkForEmptyList();
		cf.title(_ylab);
	} // end yaxis
	public void axes(String _xlab, String _ylab){
		checkForEmptyList();
		cf.axes(_xlab, _ylab);
	} // end axes

	public void cmap(String map){
		checkForEmptyList();
		cf.setCmap(map);
		cf.update(); 	
	} // end cmapt 

	public void setCloseAction(String action){
		Iterator<Integer> keySetIterator = figureList.keySet().iterator(); 
		while(keySetIterator.hasNext()){
			figureList.get(keySetIterator.next()).setCloseAction(action); 
		}
	} // end setCloseAction

	public void legend(String... leglabel){
		checkForEmptyList(); 
		cf.setLegendLabels(leglabel);
	} // end legend 

	public void addToLegend(String s){
		checkForEmptyList();
		cf.addToLegend(s);
	} // end addToLegend

	//********** Histograms 
		public void hist(double[] x, int nbins, Color color){
			retFigure(); 
			cf.hist(x, nbins, color);
		} // end hist 

		public void hist(double[] x, int nbins){
			retFigure(); 
			cf.hist(x,nbins, cf.getColor()[0]);
		} // end hist(x,nbins)

		public void hist(double[] x){ hist(x, 10); } // end hist(x)
		public void hist(double[] x, Color color) {hist(x,10,color);} // end hist(x,color)

	//********** Scatter Plots 
		public void scatter(double[] x, double[] y, Color color){
			retFigure();
			cf.scatter(x,y,color); 
		} // end scatter 

		public void scatter(double[] x, double[] y){
			retFigure();
			cf.scatter(x,y,cf.getColor()[0]); 
		} // end scatter 

	//********** Line Plots 
		public void plot(double[] x, double[] y, Color mcolor, Color lcolor ){
			retFigure(); 
			cf.plot(x,y,mcolor,lcolor);
		} // end plot 
		
		public void plot(double[] x, double[] y, Color color ){ plot(x,y,color,color); } // end plot 

		public void plot(double[] x, double[] y){
			retFigure(); 
			Color[] colors = cf.getColor();
			cf.plot(x,y,colors[0], colors[1]);
		} // end plot 

	 //********* Heat Map 
	 	public void heatmap(double[][] x, double xmin, double xmax, double ymin, double ymax ){
	 		retFigure();
	 		cf.heatmap(x, xmin, xmax, ymin, ymax);
	 	} // end heat map 	


	//********** private helper methods 

	private void checkForEmptyList(){ // set CurrentFigure 
		if(figureList.isEmpty()){
			figureList.put(new Integer(0), new Figure());
			cf = figureList.get(0);
			return;
		}
	} // checkForEmptyList


	private void retFigure(){
		// If hold is on, return 
		if(this.HOLD){ 
		checkForEmptyList(); 
		return; 
		} 
		// Else, add new figure and switch to it 
		this.setNextFigureIndex(); 
		figureList.put(new Integer(nextfigure), new Figure());
		cf = figureList.get(nextfigure);
	} // end retFigure

	




///////////////////
//****************
//---------------- Inner Class: Figure 
//****************
//////////////////


	
public class Figure{

		DrawingFrame frame = new PlotFrame("x", "y","Figure");
		PlottingPanel panel; 
		ArrayList<Dataset> DataList = new ArrayList<Dataset>();
		Color MarkerColor = new Color(255,0,0,255);
		Color LineColor = new Color(0,255,0,255);
		boolean visibility = true; 
		String cmap = "rainbow"; // default cmap 
		JInternalFrame LegendFrame = null;
		Legend l = null;

		//******** Default Constructor 
		private Figure(){ 
			// Set Frame Parameters
			frame.setSize(500,500);		
			// Set Content Pane 
			JDesktopPane desktop = new JDesktopPane();
			frame.setContentPane(desktop);
			desktop.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);
			// Create Internal Frame and add to Top-Level JFrame 
			JInternalFrame iframe = new JInternalFrame("Plot", true);
			frame.add(iframe); 
			
			// Create Plotting Panel and add to Internal Frame 
			panel = new PlottingPanel("x", "y", "Plot");
		 	iframe.add(panel);
		 	iframe.pack();
			iframe.setVisible(visibility);

			// Set Close Action 
			setCloseAction(CloseAction);

			// Set Top-Level Visibility 
			frame.setVisible(visibility);

			// Maximize Internal Plot Frame 
			try{
				iframe.setMaximum(true);
			}catch(PropertyVetoException e ){
				System.out.println("PropertyVetoException: Could not maximize internal frame.");
			}
			
		} // end constructor


		//********* Constructor 
		private Figure(String xlabel, String ylabel, String title){
			// Set Frame Parameters
			frame.setSize(500,500);
			// Set Content Pane 
			frame.setContentPane(new JDesktopPane());
			// Create Internal Frame and add to Top-Level JFrame 
			JInternalFrame iframe = new JInternalFrame("Plot", true);
			frame.add(iframe); 
			
			// Create Plotting Panel and add to Internal Frame 
			panel = new PlottingPanel(xlabel,ylabel,title);
		 	iframe.add(panel);
		 	iframe.pack();
			iframe.setVisible(visibility);

			// Set Close Action 
			setCloseAction(CloseAction);

			// Set Top-Level Visibility 
			frame.setVisible(visibility);

			// Maximize Internal Plot Frame 
			try{
				iframe.setMaximum(true);
			}catch(PropertyVetoException e ){
				System.out.println("PropertyVetoException: Could not maximize internal frame.");
			}
			
		} // end constructor 


		//-------------------- Constructors 
		//********************
		//-------------------- Setters and Getters  



		public void setVisible(boolean _visibility) {
			visibility = _visibility; 
			frame.setVisible(visibility);
			Component[] comps = frame.getComponents();
			for(Component c : comps){
				if(c instanceof JFrame) c.setVisible(visibility);
			} // end for loop 
		}
		public boolean isVisible() { return visibility; }
		public void update(){ this.panel.repaint(); }
		public Color[] getColor() {return new Color[] {MarkerColor, LineColor}; }
		public void title(String _title){ frame.setTitle(_title);}
		public void xaxis(String _xlab){ frame.setXLabel(_xlab);}
		public void yaxis(String _ylab){ frame.setYLabel(_ylab);}
		public void axes(String _xlab, String _ylab){ xaxis(_xlab); yaxis(_ylab); }
		public void setCloseAction(String CloseAction){
			if(CloseAction.equalsIgnoreCase("exit")) frame.setDefaultCloseOperation(javax.swing.JFrame.EXIT_ON_CLOSE);   
			else if(CloseAction.equalsIgnoreCase("dispose")) frame.setDefaultCloseOperation(javax.swing.JFrame.DISPOSE_ON_CLOSE);
			else if(CloseAction.equalsIgnoreCase("nothing")) frame.setDefaultCloseOperation(javax.swing.JFrame.DO_NOTHING_ON_CLOSE);
			else if(CloseAction.equalsIgnoreCase("hide")) frame.setDefaultCloseOperation(javax.swing.JFrame.HIDE_ON_CLOSE);
		} // end setCloseAction

		//-------------------- Setters and Getters 
		//********************
		//-------------------- Plot Methods  
		
		
		public void hist(double[] data, int nbins, Color color){

			// Find min and max
			double min = data[0];
			double max = data[0];
			for(int i = 0; i < data.length; i++){
				if(data[i] < min){
					min = data[i];
				}
				if(data[i] > max){
					max = data[i]; 
				}
			} // min-max loop
			
			// Determine bin start values 
			double[] bins = new double[nbins];
			for(int i = 0; i < nbins; i++){
				bins[i] = min + i*(max-min)/(double)nbins;
			}
			
			// Sort data into bins
			double[] freq = new double[bins.length];
			for(int i = 0; i < data.length; i++){
				int cbin = 0;
				while(data[i] > bins[cbin] && cbin < bins.length-1){
					cbin++;
				}
				freq[cbin--]++;
			} // end for sort loop
			
			addData("hist", bins, freq, color);
	
		} // end hist() 
		
		public void scatter(double[] x, double[] y, Color color){
			addData("scatter", x, y, color );
		} // end scatter 

		public void plot(double[] x, double[] y, Color mcolor, Color lcolor){
			addData("plot", x, y, mcolor, lcolor);
		} // end plot 

		public void heatmap(double[][] x, double xmin, double xmax, double ymin, double ymax){
			GridPlot plot = new GridPlot();
			plot.setAll(ArrayHelper.invert(x,1), xmin, xmax, ymin, ymax);
			panel.addDrawable(plot);	
		} 


		private void addData(String type, double[] x, double[] y, Color... VarColors){
			Dataset data = new Dataset(); 
			panel.addDrawable(data);
			DataList.add(data);
			data.append(x,y); 
			if(type.equalsIgnoreCase("plot")){
				data.setConnected(true); 
				data.setMarkerShape(Dataset.CIRCLE);
				data.setMarkerColor(VarColors[0]);
				data.setLineColor(VarColors[1]);
			}else if(type.equalsIgnoreCase("scatter")){
				data.setConnected(false);
				data.setMarkerShape(Dataset.CIRCLE);
				data.setMarkerColor(VarColors[0]);
			}else if(type.equalsIgnoreCase("hist")){
				data.setMarkerShape(Dataset.BAR);
				data.setMarkerColor(VarColors[0]);
			}

			// If cmap set, update 
			if(cmap != null) this.setCmap(cmap);			
			// If figure is visible, repaint 
			if(this.isVisible()) this.update();
		} // end addData 
		
		

		//-------------------- Plot methods 
		//********************
		//-------------------- Color methods 

		//********** setCmap
		public void setCmap(String map){
			int ncolors = DataList.size();
			Color[] colors = new Color[ncolors];
			if(map == "rainbow"){
				colors = this.getRainbow(ncolors); 
				cmap = map; 
			} else if(map == "random"){ 
				for(int i = 0; i < ncolors; i++){ colors[i] = getRandomColor(); } // end for
				cmap = map;  
			} else{  
				for(int i = 0; i < ncolors; i++){ colors[i] = MarkerColor;} // end for loop 
				System.out.println("Cmap not recognized. Using default.");	
				cmap = null; 
			} // end else  
			
			for(int i = 0; i < ncolors; i++){
				DataList.get(i).setMarkerColor(colors[i]);	
				DataList.get(i).setLineColor(colors[i]);
			} // end loop 
		} // end setCmap 

		//************** Random color
		public Color getRandomColor(){
			Random random = new Random(); 
			int[] color = new int[3];
			for(int i = 0; i < 3 ; i++){
			  color[i] = (int) (255*random.nextDouble());
			}
			Color c = new Color(color[0],color[1],color[2],255);
			return c;	
		} // end random color method 
		
		//***************  Get Rainbow
		public Color[] getRainbow(int numcolors){
			// Returns rainbow color map	
			Color[] cmap = new Color[numcolors];
				for(int i = 0; i < numcolors; i++){
					float h = (i+1)*(1/(float)numcolors);
					cmap[i] = Color.getHSBColor(h,1,1);
				}		
			return cmap;
		}


		//-------------------- Color methods 
		//********************
		//-------------------- Legend methods 

			// *Only way to update legend should be through changing label list 
		
		public void setLegendLabels(String... labels){
			createLegend(labels);
      Color[] cmap = new Color[DataList.size()];
      for(int i = 0; i < DataList.size(); i++){
        cmap[i] = DataList.get(i).getFillColor();
      }
      l.setCmap(cmap);
      l.setLegendLabels(labels);
		} // end setLegend

    public void createLegend(String... labels){
      if(this.LegendFrame == null ){
        LegendFrame = new JInternalFrame("Legend", true);
        LegendFrame.setMinimumSize(new Dimension(100,80));
        Dimension d = frame.getSize(); 
        LegendFrame.setLocation((int)(0.7*d.getWidth()), (int) (d.getHeight()/2));
        LegendFrame.setSize(100,100);
        LegendFrame.toFront();
        frame.add(LegendFrame);
        l = new Legend(labels);
        LegendFrame.add(l);
        LegendFrame.pack();
        LegendFrame.setVisible(visibility);
      }
    }
		public void addToLegend(String s){
			createLegend(s);
      Color[] cmap = new Color[DataList.size()];
      for(int i = 0; i < DataList.size(); i++){
        cmap[i] = DataList.get(i).getFillColor();
      }
      l.setCmap(cmap);
      l.addToLegend(s);
		} // addToLegend 

		


} // end inner class (Figure)



} // end class Plotter 
