package custom.io;

import java.io.File; 
import java.io.FilenameFilter; 
import java.util.List;
import java.util.ArrayList;
import java.util.Arrays; 
import java.util.Collection;

public class FilenameParser{

List<String> includeList = new ArrayList<String>(); 
List<String> excludeList = new ArrayList<String>(); 
File path = new File("."); 

public FilenameParser(String... filterList){
	this.setInclusion(filterList);
} // constructor


public void clear(){
  includeList.clear();
  excludeList.clear();
}

public void setPath(String _path){
	path = new File(_path); 
} // end setPath

public void printFilter(){

for(String str : includeList){ System.out.println("Include: " + str);
	}

}

public void printPath() { System.out.println(path.toString());}


public File[] getFiles(){
	File[] filenames = path.listFiles(new Filter() ); 
	Arrays.sort(filenames);
	return filenames;
} // end getFiles 

public File[] getFiles(String _path){
	setPath(_path);
	File[] files = getFiles();
	return files; 
} // end getFiles(string)

public void removeFilter(String filter){
	if(includeList.contains(filter)) includeList.remove(filter);
	if(excludeList.contains(filter)) excludeList.remove(filter);	
} // end clearFilter

//********** Setters 

public void setInclusion(String... varargs){
	// clear filter 
	includeList.clear(); 
	// add new filters 
	for(int i = 0; i < varargs.length; i++){
		includeList.add(varargs[i]);
	} // end for loop 
} // end setFilter

public void setExclusion(String... varargs){
	excludeList.clear();
	for(int i = 0; i < varargs.length; i++ ){
		excludeList.add(varargs[i]);
	} // end for loop 

} // end setExclusion


//********** Include Methods 

public void include(String addfilter){
	includeList.add(addfilter);
	if(excludeList.contains(addfilter)) removeExclusion(addfilter);
} // include

public void include(String... varargs){
	for(String str : varargs){
		include(str);
	} // end for 
} // end include 

public void include(Collection<String> strlist){
	for(String str : strlist){
		include(str);
	}
} // end include 


//********* Exclude Methods 

public void exclude(String exfilter){
	excludeList.add(exfilter);
	if(includeList.contains(exfilter)) removeInclusion(exfilter);
} // exclude 

public void exclude(String... varargs){
	for(String str : varargs){
		exclude(str);
	} // end for 
} // end exclude 

public void exclude(Collection<String> strlist){
	for(String str : strlist){
		exclude(str);
	}
} // end exclude 


//********* Private Helper Methods 


private void removeInclusion(String remfilter){
	includeList.remove(remfilter);
} // removeFilter

private void removeExclusion(String remexfilter){
	excludeList.remove(remexfilter);
} // removeExclusion 


//private File[] sortNames(File[] files){
//	// bubble sort of filenames 

//} // 



	//*********** Inner class: Filter 

	class Filter implements FilenameFilter{

		public Filter(){}		
		
		public boolean accept(File dir, String name){
			// return true if name contains all strings in inclusive list, and none in exclusion list  
			// return false otherwise, of if name contains any string in exclude list
			
			for(String ex : excludeList){
				if(name.contains(ex)) return false; 
			} // end ex loop 

			for(String in : includeList){
				if(! name.contains(in)) return false;
			} // end in loop 

			return true; 
			
			
		} // end accept 
		
		
	} // end inner class filter 


} // end class 











