package diffusion;

import org.opensourcephysics.numerics.LUPDecomposition;
import org.opensourcephysics.numerics.VectorMath;

public class VoronoiDiagram {
double[] center;
double[][] vertex;
double[][] projection;  //neighbor projections
double[] normal;
double[] d;
double[] s;
double area;
double GTP;  //concentration of GTP unlabeled
double GDP;
double GAPlabeled;
double GAP;
double totGAP;
int FRAP;


VoronoiDiagram(double[] center, double[] normal, double[][] projection){
	this.center=center;
	this.normal=normal;
	this.projection=projection;
	vertex= new double[projection.length][3];
	s= new double[projection.length];
	d= new double[projection.length];
}

public double[][] getVoronoiDiagramVertices(){
	int numOfNeighbors=projection.length;
	for(int t=0;t<numOfNeighbors;t++){
	double[] v=new double[3];   // a vector between the surface node and one of the neighbors
	double[] v1=new double[3];  // a vector between the surface node and the next neighbor
	v[0]=projection[t][0]-center[0];
	v[1]=projection[t][1]-center[1];
	v[2]=projection[t][2]-center[2];
	//System.out.println("vx="+v[0]+"	vy="+v[1]+"	vz="+v[2]);
	if(t!=numOfNeighbors-1){
		v1[0]=projection[t+1][0]-center[0];
		v1[1]=projection[t+1][1]-center[1];
		v1[2]=projection[t+1][2]-center[2];
	}
	else{
		v1[0]=projection[0][0]-center[0];
		v1[1]=projection[0][1]-center[1];
		v1[2]=projection[0][2]-center[2];
	}
	double[] midPoint=new double[3];
	double[] midPoint1=new double[3];
	midPoint[0]=center[0]+v[0]/2;
	midPoint[1]=center[1]+v[1]/2;
	midPoint[2]=center[2]+v[2]/2;
	midPoint1[0]=center[0]+v1[0]/2;
	midPoint1[1]=center[1]+v1[1]/2;
	midPoint1[2]=center[2]+v1[2]/2;
	/*System.out.println("------------------------------------------------------------------------------------------------------>");
	System.out.println("pos[0]="+center[0]+"		pos[1]="+center[1]+"		pos[2]="+center[2]);
	System.out.println("posNeighbor["+t+"][0]="+projection[t][0]+"		posNeighbor["+t+"][1]="+projection[t][1]+"		posNeighbor["+t+"][2]="+projection[t][2]);
	if(t!=numOfNeighbors-1)
		System.out.println("posNeighbor["+(t+1)+"][0]="+projection[t+1][0]+"		posNeighbor["+(t+1)+"][1]="+projection[t+1][1]+"		posNeighbor["+(t+1)+"][2]="+projection[t+1][2]);
	else
		System.out.println("posNeighbor1[0]="+projection[0][0]+"		posNeighbor1[1]="+projection[0][1]+"		posNeighbor1[2]="+projection[0][2]);
	System.out.println("v[0]="+v[0]+"		v[1]="+v[1]+"		v[2]="+v[2]);
	System.out.println("v1[0]="+v1[0]+"		v1[1]="+v1[1]+"		v1[2]="+v1[2]);
	System.out.println("midPoint[0]="+midPoint[0]+"		midPoint[1]="+midPoint[1]+"		midPoint[2]="+midPoint[2]);
	System.out.println("midPoint1[0]="+midPoint1[0]+"		midPoint1[1]="+midPoint1[1]+"		midPoint1[2]="+midPoint1[2]);*/
	v=VectorMath.normalize(v);
	v1=VectorMath.normalize(v1);
	double[] u=new double[3];
	u=VectorMath.cross3D(normal, v);
	double[] u1=new double[3];
	u1=VectorMath.cross3D(normal, v1);
	/*System.out.println("---------------------->			numOfNeighbor="+t);
	System.out.println("v_norm[0]="+v[0]+"		v_norm[1]="+v[1]+"		v_norm[2]="+v[2]);
	System.out.println("v1_norm[0]="+v1[0]+"		v1_norm[1]="+v1[1]+"		v1_norm[2]="+v1[2]);
	System.out.println("norm[0]="+normal[0]+"		norm[1]="+normal[1]+"		norm[2]="+normal[2]);
	System.out.println("u_norm[0]="+u[0]+"		u_norm[1]="+u[1]+"		u_norm[2]="+u[2]);
	System.out.println("u1_norm[0]="+u1[0]+"		u1_norm[1]="+u1[1]+"		u1_norm[2]="+u1[2]);*/
	double[][] coefficients=new double[2][2];  //we have 3 equations and two unknowns for the intersection point coordinates in 3d each two equations that have a non-zero determinant can be used to find the unknowns 
	double[] result={0.,0.};
	for(int i=0;i<3;i++){
		for(int j=0;j<3;j++){
			if(i<j){
				double[][] matrix={{u[i],-1*u1[i]},{u[j],-1*u1[j]}};
				double determinant=matrix[0][0]*matrix[1][1]-matrix[0][1]*matrix[1][0];
				/*System.out.println("i="+i+"		j="+j);
				System.out.println("matrix[0][0]="+matrix[0][0]+"	matrix[0][1]="+matrix[0][1]+"	matrix[1][0]="+matrix[1][0]+"	matrix[1][1]="+matrix[1][1]);
				System.out.println("determinant="+determinant);*/
				if(Math.abs(determinant)>Math.pow(10,-5) && (Math.abs(midPoint1[i]-midPoint[i])>Math.pow(10,-5) || Math.abs(midPoint1[j]-midPoint[j])>Math.pow(10,-5))){
					coefficients=matrix;
					result[0]=midPoint1[i]-midPoint[i];
					result[1]=midPoint1[j]-midPoint[j];
					/*System.out.println("matrix[0][0]="+matrix[0][0]+"	matrix[0][1]="+matrix[0][1]+"	matrix[1][0]="+matrix[1][0]+"	matrix[1][1]="+matrix[1][1]);
					System.out.println("determinant is nonzero and midPoint1["+i+"]-midPoint["+i+"]="+result[0]+"		midPoint1["+j+"]-midPoint["+j+"]="+result[1]);*/
				}
			}
		}
	}	
	/*if(result[0]==0. && result[1]==0.){
		System.out.println("Error occured in VoronoiDiagram method when finding intersection point of two lines!!!!");
	}*/
	LUPDecomposition LUDecomposition=new LUPDecomposition(coefficients);
	double[] linesParameter=LUDecomposition.solve(result);
	double[] intersectionPoint=new double[3];
	/*System.out.println("num of neighbor="+t);
	System.out.println("t="+linesParameter[0]+"		t_prime="+linesParameter[1]);*/
	for(int i=0;i<3;i++){
	intersectionPoint[i]=midPoint[i]+u[i]*linesParameter[0];
	if(Math.abs(intersectionPoint[i]-midPoint1[i]-u1[i]*linesParameter[1])>Math.pow(10,-5))
		System.out.println("there has been an error in finding the intersection of lines in getVoronoiDiagramVertices");
	/*System.out.println("intersectionPoint["+i+"]="+"midPoint["+i+"]+u["+i+"]*linesParameter[0]="+midPoint[i]+"+"+u[i]+"*"+linesParameter[0]+"="+intersectionPoint[i]);
	System.out.println("intersection point x["+i+"]="+intersectionPoint[i]);*/
	}
	vertex[t]=intersectionPoint;
	}
	return vertex; 
}

public void Geodesic(){
	for(int i=0;i<projection.length;i++){
		if(i!=projection.length-1)
			s[i+1]=Math.sqrt(Math.pow((vertex[i][0]-vertex[i+1][0]),2)+Math.pow((vertex[i][1]-vertex[i+1][1]),2)+Math.pow((vertex[i][2]-vertex[i+1][2]),2));
		else
			s[0]=Math.sqrt(Math.pow((vertex[i][0]-vertex[0][0]),2)+Math.pow((vertex[i][1]-vertex[0][1]),2)+Math.pow((vertex[i][2]-vertex[0][2]),2));	
	}
	for(int i=0;i<projection.length;i++){
		d[i]=Math.sqrt(Math.pow((projection[i][0]-center[0]),2)+Math.pow((projection[i][1]-center[1]),2)+Math.pow((projection[i][2]-center[2]),2));
	}
}
public double distanceFromTheCloserTip(double length,double radius){
	double z;
	if(center[2]>length/2-radius)
		z=radius*Math.acos(normal[2]);
	else if(center[2]<-length/2+radius)
		z=(length-2*radius)+(radius*Math.PI/2)+(radius*Math.PI/2-radius*Math.acos(Math.abs(normal[2])));
	else
		z=Math.PI/2*radius+(length/2-radius-center[2]);
	if(z>(Math.PI/2*radius+(length/2-radius)))
		z=(length-2*radius+Math.PI*radius)-z;
	return z;
}

public double distanceFromTheLowerTip(double length,double radius){
	double z;
	if(center[2]<0)
		z = distanceFromTheCloserTip(length,radius);
	else
		z = (Math.PI-2)*radius+length-distanceFromTheCloserTip(length,radius);
	return z;
}

public double phi(){
	double x;
	x=Math.atan(Math.abs(normal[1]/normal[0]));
	return x;
}
}
