package simulation;

////// In this code I have a save data file for all the nodes coordinates to re create the voronoi diagrams later
import org.opensourcephysics.controls.AbstractSimulation;
import org.opensourcephysics.controls.SimulationControl;
import diffusion.*;

public class App extends AbstractSimulation {

	private double dt;
	private double length, radius, DGTP, DGDP,Smin,Smax,DGAP,k0p,k1p,k2p,k1n,k2n,k3n,c_sat,r_GAP,T,visualizationTime,r_GTP,r_GDP,r_noise;
	private int h,nx,ny,nz,totNumNodes;
	private double maxGTP, initialGTP, initialGDP;
	private boolean showAnimation;
	private double lx,ly,lz;
	private model model = new model();
	
	public void initialize(){
		this.delayTime = 0;
		
		DGTP= control.getDouble("DGTP");
		DGDP= control.getDouble("DGDP");
		DGAP= control.getDouble("DGAP");
		length= control.getDouble("length");
		radius= control.getDouble("radius");
		k0p= control.getDouble("k0p");
		k1p= control.getDouble("k1p");
		k2p= control.getDouble("k2p");
		k1n= control.getDouble("k1n");
		k2n= control.getDouble("k2n");
		k3n= control.getDouble("k3n");
		r_noise= control.getDouble("r_noise");
		h= control.getInt("h");
		c_sat= control.getDouble("c_sat");
		r_GDP= control.getDouble("r_GDP");
		r_GTP= control.getDouble("r_GTP");
		r_GAP= control.getDouble("r_GAP");
		T= control.getDouble("T");
		visualizationTime= control.getDouble("visualizationTime");
		Smin = control.getDouble("Smin");
		Smax = control.getDouble("Smax");
		showAnimation = control.getBoolean("showAnimation");
		nx= control.getInt("nx"); //number of nodes in each row
		ny= control.getInt("ny");
		nz= control.getInt("nz");
		dt= control.getDouble("dt");
		lx=2*radius/nx;
		ly=2*radius/ny;
		lz=length/nz;
		control.println("lx="+lx+"			ly="+ly+"		lz="+lz);
		setModelParameters();
		//model.initialize();
		//Geometry();  //plots the geometry of a fission yeast cell
		model.surfaceNodeInit();  //creates nodes on the center of each surface of each cube that belongs to the mesh network
		model.geometryStatusInit(); //determines which nodes belong to the cubes that are inside the geometry of the cell (pixelated geometry)
		//model.surfacePoints();  //shows the surface nodes that create the mesh network
		model.concentrationStatusInit(); //determines which surface nodes show the surface of the pixelated geometry
		model.initializeNodeNumbers();
		model.findingNeighbors();  //Finding normal vector to the real surface at the closest surface point for each surfaceNode and finding neighbors for each surfaceNode (neighbors are all those nodes that they facets hasat least one common point with the facet of the surfaceNode that is being checked )
		model.symmetrization();
		model.calculationOfVoronoiDiagramsArea();
		model.savingDataInit();
		model.concentrationInit();
		model.concentrationPlot();
		double pixel=Math.min(Math.min(lx, ly),lz);
		if(dt>(pixel*pixel/(4*DGDP)) || dt>(pixel*pixel/(4*DGTP)) || dt>(pixel*pixel/(4*DGAP))){
			System.out.println("dt is large!!!!");
			if(dt>(pixel*pixel/(4*DGDP)))
				System.out.println("make it smaller than:"+DGDP+"value===="+(pixel*pixel/(4*DGDP)));
			else if(dt>(pixel*pixel/(4*DGTP)))
				System.out.println("make it smaller than:"+DGTP+"value===="+(pixel*pixel/(4*DGTP)));
			else
				System.out.println("make it smaller than:"+DGAP+"value===="+(pixel*pixel/(4*DGAP)));
		}
	}
	
	protected void doStep(){

		// If model finished, return 
			if(model.isFinished()){
			stopSimulation();	
			}
		// Print Time 
			control.println("Time: " + model.getTime()+ "/" + T); 
		// Do Model Step 
			model.doStep(showAnimation); 
		
	} // end doStep()
	
	public void stop(){
		model.closingCommands();
		// Print Stop Time 
		control.println("Stopped at " + model.getTime());
 
	}
	
	//Buttons
	public void reInitParams(){
		DGTP= control.getDouble("DGTP");
		DGDP= control.getDouble("DGDP");
		DGAP= control.getDouble("DGAP");
		k0p= control.getDouble("k0p");
		k1p= control.getDouble("k1p");
		k2p= control.getDouble("k2p");
		k1n= control.getDouble("k1n");
		k2n= control.getDouble("k2n");
		k3n= control.getDouble("k3n");
		r_GDP= control.getDouble("r_GDP");
		r_GTP= control.getDouble("r_GTP");
		c_sat= control.getDouble("c_sat");
		r_GAP= control.getDouble("r_GAP");
		r_noise= control.getDouble("r_noise");
		h= control.getInt("h");
		dt= control.getDouble("dt");
		T= control.getDouble("T");
		showAnimation= control.getBoolean("showAnimation");
		visualizationTime= control.getDouble("VisualizationTime");
		model=new model();
	}

	public void reset(){
		control.setValue("DGTP",0.04);  //micrometer^2/s
		control.setValue("DGDP",0.15);  //micrometer^2/s
		control.setValue("DGAP",0.2);
		control.setValue("k0p", 0.02);
		control.setValue("k1p", 5);
		control.setValue("k2p", 1000);
		control.setValue("k1n", 0.002);
		control.setValue("k2n",0.1);
		control.setValue("k3n", 100);
		control.setValue("r_GDP", 0.001);
		control.setValue("r_GTP", 0.02);
		control.setValue("r_GAP", 0.1);
		control.setValue("c_sat", 60);  //1
		control.setValue("h", 2);
		control.setValue("r_noise", 0.002);
		control.setValue("length",6.0); //micrometer
		control.setValue("radius",2.0); //micrometer
		control.setValue("dt", 0.01);  //s
		control.setValue("nx", 20);
		control.setValue("ny", 20);
		control.setValue("nz", 45);
		control.setValue("Smin", 4);
		control.setValue("Smax", 6);
		control.setValue("T", 600);
		control.setValue("showAnimation", true);
		control.setValue("visualizationTime", 0.5);
		model=new model();
	}
	
	
	private void setModelParameters(){ 
		model.setParameter("DGTP", DGTP);
		model.setParameter("DGDP", DGDP);
		model.setParameter("DGAP", DGAP);
		model.setParameter("k0p", k0p);
		model.setParameter("k1p", k1p);
		model.setParameter("k2p", k2p);
		model.setParameter("k1n", k1n);
		model.setParameter("k2n", k2n);
		model.setParameter("k3n", k3n);
		model.setParameter("r_GDP", r_GDP);
		model.setParameter("r_GTP", r_GTP);
		model.setParameter("c_sat", c_sat);
		model.setParameter("h", h);
		model.setParameter("r_noise", r_noise);
		model.setParameter("r_GAP", r_GAP);
		model.setParameter("length", length);
		model.setParameter("radius", radius);
		model.setParameter("dt", dt);
		model.setParameter("nx", nx);
		model.setParameter("ny", ny);
		model.setParameter("nz", nz);
		model.setParameter("Smin", Smin);
		model.setParameter("Smax", Smax);
		model.setParameter("T", T);
		model.setParameter("visualizationTime", visualizationTime);
		model.setParameter("lx", lx);
		model.setParameter("ly", ly);
		model.setParameter("lz", lz);
		model.ShowGraphics(showAnimation);
		
	}
	public static void main(String[] args){
		SimulationControl control= SimulationControl.createApp(new App());
	}

}
